#素材
class ResourcePicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  belongs_to :picture
  has_many :panel_pictures
  belongs_to :original_picture
  
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :artist_id, :presence => true, :numericality => true, :existence => true
  validates :license_id, :presence => true, :numericality => true, :existence => true
  validates :original_picture_id, :presence => true, :numericality => true, :existence => true
  validates :artist_name, :presence => true
  validates :classname, :presence => true, :length => {:maximum => 50}
  validates :picture_id, :presence => true, :numericality => true, :existence => true
  
  before_destroy :destroy_with_file
  
  def destroy_with_file
    PictureIO.resource_picture_io.delete self.filename
    PictureIO.resource_picture_io.class.subdirs.each do |d|
      next if d.empty?
      PictureIO.resource_picture_io.delete(self.filename, d) if PictureIO.resource_picture_io.exist?(self.filename, d)
    end
  end
  
  def self.resize(data, dw, dh)
    Magick::Image.from_blob(data).shift.resize(dw, dh)
  end
  
  #サイズの調整（limw,limhに必ず収まるように合わせる）
  def self.fix_size_both(limw, limh, w, h)
    wr = if w > limw
      limw*100/w
    else
      100
    end
    hr = if h > limh
      limh*100/h
    else
      100
    end
    res = if wr < hr
      #幅の方が圧縮率が高い
      [w*wr/100, h*wr/100]
    else
      #高さの方が圧縮率が高い
      [w*hr/100, h*hr/100]
    end
    res
  end
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def gifname
    "#{self.id}.gif"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def url subdir = nil
    '/resource_pictures/' + (subdir.to_s.empty? ? '' : subdir.to_s + '/' ) + filename
  end
  
  def thumbnail(rimg)
    tw, th = ResourcePicture.fix_size_both(MagicNumber['thumbnail_width'], MagicNumber['thumbnail_height'], rimg.columns, rimg.rows)
    ResourcePicture.resize(rimg.to_blob, tw, th).to_blob
  end
  
  def opt_img_tag
    tw, th = ResourcePicture.fix_size_both(MagicNumber['thumbnail_width'], MagicNumber['thumbnail_height'], self.width, self.height)
    {:src => self.url, :width => tw, :height => th}
  end
  
  def copy_data(op)
    attr = {:width => op.width, :height => op.height, :ext => op.ext, :filesize => op.filesize, 
      :original_picture_id => op.id, :artist_id => op.artist_id, :md5 => op.md5
    }
    self.attributes = attr
  end
  
  def data_to_mgk picture_data
    begin
      mgk = Magick::Image.from_blob(picture_data).shift
    rescue 
      self.errors.add :base, 'magick failed'
      return false
    end
    mgk
  end
  
  def op_mgk
    d = self.original_picture.restore
    return false unless d
    self.data_to_mgk d
  end
  
  def new_picture mgk
    pc = Picture.new
    pc.copy_data self
    pc.store mgk
    pc
  end
  
  
  def to_gif?
    self.dext == 'png' and self.flag_gif_convert >= 0
  end
  
  def self.png_to_gif(data)
    res = nil
    begin
      mgk = Magick::Image.from_blob(data).shift
      mgk.format = 'gif'
      res = mgk
    rescue
      res = false
    end
    res
  end
  
  def store
    res = false
    self.copy_data self.original_picture
    mgk = self.op_mgk
    return false unless mgk
    OriginalPicture.transaction do
      pc = self.new_picture mgk
      if res = pc.valid?
        self.picture_id = pc.id
        if res = self.save
          res = self.store_picture_with_gif(mgk)
        end
      else
        self.errors.add :base, 'picture does not create'
      end
      raise ActiveRecord::Rollback unless res
    end
    res
  end
  
  def store_picture_with_gif(mgk)
    if res = self.store_picture(mgk, self.filename)
      if self.to_gif?
        if gifmgk = ResourcePicture.png_to_gif(mgk.to_blob)
          res = self.store_picture(gifmgk, self.gifname)
        else
          self.errors.add :base, 'picture data can not conv to gif'
          res = false
        end
      end
    else
      self.errors.add :base, 'resource picture io does not work'
    end
    res
  end
  
  def store_picture(mgk, fn)
    res = false
    tdata = self.flag_thumbnail >= 0 ? thumbnail(mgk) : mgk.to_blob
    fdata = mgk.to_blob
    return false unless PictureIO.resource_picture_io.put(tdata, fn)
    PictureIO.resource_picture_io.put(fdata, fn, 'full')
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    ResourcePicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => [:license, :artist]}
  end
  
  def self.list_json_opt
    {:include => [:license, :artist]}
  end
  
  def self.show rid, opt = {}
    r = ResourcePicture.find(rid, :include => self.show_include_opt(opt))
#    raise ActiveRecord::Forbidden unless c.visible?(au)
    r
  end
  
  def self.show_include_opt opt = {}
    res = [:license, :artist]
    res.push(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => [:license, :artist]}
  end
  
  def self.visible_count
    ResourcePicture.count
  end
  
  def credit_template
    "#{self.classname.tableize}/attributes/credit"
  end
  
  def credit_data
    begin
      @credit_data = JSON.parse(self.credit) unless @credit_data
    rescue 
    end
    @credit_data
  end
  
  def flags
    begin
      @flags = JSON.parse(self.settings) unless @flags
    rescue 
    end
    @flags
  end
  
  def flags=(s)
    @flags = s
  end
  
  def flag_open
    @flag_open = flags["open"] unless @flag_open
    @flag_open
  end
  
  def flag_commercial
    @flag_commercial = flags["commercial"] unless @flag_commercial
    @flag_commercial
  end
  
  def flag_official
    @flag_official = flags["official"] unless @flag_official
    @flag_official
  end
  
  def flag_attribution
    @flag_attribution = flags["attribution"] unless @flag_attribution
    @flag_attribution
  end
  
  def flag_derive
    @flag_derive = flags["derive"] unless @flag_derive
    @flag_derive
  end
  
  def flag_thumbnail
    @flag_thumbnail = flags["thumbnail"] unless @flag_thumbnail
    @flag_thumbnail
  end
  
  def flag_gif_convert
    @flag_gif_convert = flags["gif_convert"] unless @flag_gif_convert
    @flag_gif_convert
  end
  
  def flag_reverse
    @flag_reverse = flags["reverse"] unless @flag_reverse
    @flag_reverse
  end
  
  def flag_resize
    @flag_resize = flags["resize"] unless @flag_resize
    @flag_resize
  end
  
  def flag_sync_vh
    @flag_sync_vh = flags["sync_vh"] unless @flag_sync_vh
    @flag_sync_vh
  end
  
  def flag_overlap
    @flag_overlap = flags["overlap"] unless @flag_overlap
    @flag_overlap
  end
  
end
